/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
/*
 * Bickley - a meta data management framework.
 * Copyright © 2008, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA
 */

#ifndef __KOZO_EXPR_H__
#define __KOZO_EXPR_H__

#include <glib.h>
#include <time.h>

typedef enum {
        /* Operate only on strings */
        KOZO_EXPR_CONTAINS,
        KOZO_EXPR_DOES_NOT_CONTAIN,

        /* The remaining types assume single values */

        /* Operate on all types */
        KOZO_EXPR_EQ,
        KOZO_EXPR_NE,

        /* Operate only on numbers */
        KOZO_EXPR_GE,
        KOZO_EXPR_GT,
        KOZO_EXPR_LE,
        KOZO_EXPR_LT
} KozoExprType;

typedef struct _KozoExpr KozoExpr;

KozoExpr *kozo_expr_new_string (KozoExprType type,
                                int          field_id,
                                const char  *cnst);
KozoExpr *kozo_expr_new_int    (KozoExprType type,
                                int          field_id,
                                gint32       cnst);
KozoExpr *kozo_expr_new_time   (KozoExprType type,
                                int          field_id,
                                time_t       cnst);

KozoExpr *kozo_expr_ref        (KozoExpr    *expr);
void      kozo_expr_unref      (KozoExpr    *expr);

#endif /* __KOZO_EXPR_H__ */
