/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
/*
 * Bickley - a meta data management framework.
 * Copyright © 2008, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA
 */

#include <string.h>
#include <glib/gthread.h>

#include "kozo-query-private.h"
#include "kozo-db-private.h"
#include "kozo-expr-private.h"
#include "kozo-index-private.h"
#include "kozo-index-query.h"
#include "kozo-pack.h"

typedef struct {
        KozoQuery parent;

        KozoIndex *index;
} KozoIndexQuery;

static void
add_result (KozoQuery       *query,
            KozoEntryArray  *array,
            KozoPackContext *context,
            const TDB_DATA   key,
            const TDB_DATA   data)
{
        kozo_pack_start (context);
        kozo_db_harvest_entry (key, data, context, query->field_ids);
        context->dest =
                obstack_alloc (&array->obstack, context->length);
        kozo_pack_finish (context);

        kozo_entry_array_append (array,
                                 context->dest, context->length);
}

static gboolean
kozo_index_query_run (KozoQuery      *query,
                      KozoEntryArray *results,
                      GError        **error)
{
        KozoIndexQuery *index_query;
        TDB_CONTEXT *index;
        TDB_DATA key, data, newkey;
        KozoPackContext context;

        index_query = (KozoIndexQuery *) query;

        index = kozo_index_get_index (index_query->index);
        g_return_val_if_fail (index != NULL, FALSE);

        kozo_pack_context_init (&context, FALSE,
                                query->n_field_ids, sizeof (guint32));

        key = tdb_firstkey (index);
        while (key.dptr) {
                data = tdb_fetch (index, key);

                /* TODO? */
                g_print ("'%s'\n", data.dptr + 20);

                free (data.dptr);
                newkey = tdb_nextkey (index, key);
                free (key.dptr);
                key = newkey;
        }

        return TRUE;
}

static gboolean
kozo_index_query_notify (KozoQuery      *query,
                         KozoEntryArray *queue,
                         const TDB_DATA  key,
                         const TDB_DATA  data,
                         guint8        **dest,
                         guint32        *length)
{
        /* TODO */
        return FALSE;
}

static void
kozo_index_query_free (KozoQuery *query)
{
        KozoIndexQuery *index_query;

        index_query = (KozoIndexQuery *) query;

        kozo_index_unref (index_query->index);
}

KozoQuery *
kozo_index_query_new (KozoIndex *index)
{
        KozoIndexQuery *index_query;
        KozoQuery *query;

        index_query = g_new (KozoIndexQuery, 1);
        query = (KozoQuery *) index_query;

        kozo_query_set_defaults (query,
                                 kozo_index_get_db (index));

        index_query->index = kozo_index_ref (index);

        query->run = kozo_index_query_run;
        query->notify = kozo_index_query_notify;
        query->free = kozo_index_query_free;

        return query;
}
