/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
/*
 * Bickley - a meta data management framework.
 * Copyright © 2008, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA
 */

#ifndef __KOZO_QUERY_PRIVATE_H__
#define __KOZO_QUERY_PRIVATE_H__

#include <stdlib.h>
#include <signal.h>

#include <tdb.h>

#include "kozo-query.h"
#include "kozo-db.h"
#include "kozo-util.h"

typedef enum {
        KOZO_QUERY_NOTIFY_ADDED,
        KOZO_QUERY_NOTIFY_CHANGED,
        KOZO_QUERY_NOTIFY_REMOVED
} KozoQueryNotifyType;

struct _KozoQuery {
        /* vtable */
        gboolean (* run)    (KozoQuery      *query,
                             KozoEntryArray *array,
                             GError        **error);

        gboolean (* notify) (KozoQuery      *query,
                             KozoEntryArray *queue,
                             TDB_DATA        key,
                             TDB_DATA        data,
                             guint8        **dest,
                             guint32        *length);

        void     (* free)   (KozoQuery      *query);

        /* variables */
        KozoDB *db;

        GSList *field_ids;
        guint n_field_ids;

        KozoOperator op;
        GSList *exprs;
        guint n_exprs;

        KozoQueryCallback entries_added;
        KozoQueryCallback entries_changed;
        KozoQueryCallback entries_removed;
        gpointer user_data;

        KozoEntryArray *added_queue;
        KozoEntryArray *changed_queue;
        KozoEntryArray *removed_queue;
        GMutex *queue_lock;

        int ref_count;
};

void kozo_query_set_defaults (KozoQuery          *query,
                              KozoDB             *db);

void kozo_query_notify       (KozoQuery          *query,
                              KozoQueryNotifyType type,
                              TDB_DATA            key,
                              TDB_DATA            data);

void kozo_query_flush        (KozoQuery          *query);

#endif /* __KOZO_QUERY_PRIVATE_H__ */
