/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
/*
 * Bickley - a meta data management framework.
 * Copyright © 2008, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA
 */

#ifndef __KOZO_QUERY_H__
#define __KOZO_QUERY_H__

#include <glib.h>

#include "kozo-expr.h"
#include "kozo-entry.h"

typedef enum {
        KOZO_OPERATOR_AND,
        KOZO_OPERATOR_OR
} KozoOperator;

typedef struct _KozoQuery KozoQuery;

typedef void (*KozoQueryCallback) (KozoQuery  *query,
                                   KozoEntry **entries,
                                   guint       n_entries,
                                   gpointer    user_data);

KozoQuery    *kozo_query_ref        (KozoQuery        *query);
void          kozo_query_unref      (KozoQuery        *query);

void          kozo_query_add_field  (KozoQuery        *query,
                                     int               field_id);
const GSList *kozo_query_get_fields (KozoQuery        *query);

void          kozo_query_set_op     (KozoQuery        *query,
                                     KozoOperator      op);
KozoOperator  kozo_query_get_op     (KozoQuery        *query);

void          kozo_query_add_expr   (KozoQuery        *query,
                                     KozoExpr         *expr);
const GSList *kozo_query_get_exprs  (KozoQuery        *query);

/* On the first time the query is run, entries_added will be emitted
 * once containing the results of the query. For any later
 * additions/changes/removals, the DB will group the events smartly.
 * Note that then the callbacks may be called from a different thread. */
void          kozo_query_set_funcs  (KozoQuery        *query,
                                     KozoQueryCallback entries_added,
                                     KozoQueryCallback entries_changed,
                                     KozoQueryCallback entries_removed,
                                     gpointer          user_data);

gboolean      kozo_query_run        (KozoQuery        *query,
                                     GError          **error);

#endif /* __KOZO_QUERY_H__ */
