/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */

#include <string.h>

#include "kozo-utf8.h"

/* These functions are based on evolution-data-server's e-util.c,
 * Copyright (C) 2003 Novell, Inc. Licensed under GNU GPL v2. */

/* TODO: optimize: match ²==2, ß==s, æ==ae, etc */

G_INLINE_FUNC char *
e_util_unicode_get_utf8 (const char *text, gunichar *out)
{
        *out = g_utf8_get_char (text);

        return (G_UNLIKELY (*out == (gunichar) -1)) ? NULL : g_utf8_next_char (text);
}

G_INLINE_FUNC gunichar
stripped_char (gunichar ch)
{
        GUnicodeType utype;

        utype = g_unichar_type (ch);

        switch (utype) {
        case G_UNICODE_CONTROL:
        case G_UNICODE_FORMAT:
        case G_UNICODE_UNASSIGNED:
        case G_UNICODE_COMBINING_MARK:
                /* Ignore those */
                return 0;
        default:
                /* Convert to lowercase, fall through */
                ch = g_unichar_tolower (ch);
        case G_UNICODE_LOWERCASE_LETTER:
                return ch;
        }
}

gunichar *
kozo_utf8_strcasestrip (const char *str)
{
        gunichar *needle;
        gunichar unival, sc;
        int nlen, normlen;
        const char *p;
        char *norm;

        /* FIXME unoptimal, too many iterations */
        norm = g_utf8_normalize (str, -1, G_NORMALIZE_DEFAULT);
        normlen = g_utf8_strlen (norm, -1);
        if (G_UNLIKELY (normlen == 0)) {
                g_free (norm);
                return NULL;
        }

        /* Over-estimates length */
        needle = g_new (gunichar, normlen + 1);

        nlen = 0;
        for (p = e_util_unicode_get_utf8 (norm, &unival);
             p && unival;
             p = e_util_unicode_get_utf8 (p, &unival)) {
                sc = stripped_char (unival);
                if (sc)
                       needle[nlen++] = sc;
        }

        g_free (norm);

        /* NULL means there was illegal utf-8 sequence */
        if (!p)
                return NULL;

        /* If everything is correct, we have decomposed, lowercase, stripped needle */
        needle[nlen] = 0;
        if (normlen == nlen)
                return needle;
        else
                return g_renew (gunichar, needle, nlen + 1);
}

const char *
kozo_utf8_strstrcasestrip (const char     *haystack,
                           const gunichar *needle)
{
        gunichar unival, sc;
        const char *o, *p, *q;
        int npos;

        /* FIXME this is unoptimal */
        /* TODO use Bayer-Moore algorithm, if faster */
        o = haystack;
        for (p = e_util_unicode_get_utf8 (o, &unival);
             p && unival;
             p = e_util_unicode_get_utf8 (p, &unival)) {
                sc = stripped_char (unival);
                if (sc) {
                        /* We have valid stripped char */
                        if (sc == needle[0]) {
                                q = p;
                                npos = 1;

                                while (needle[npos]) {
                                        q = e_util_unicode_get_utf8 (q, &unival);
                                        if (!q || !unival)
                                                return NULL;

                                        sc = stripped_char (unival);
                                        if ((!sc) || (sc != needle[npos]))
                                                break;

                                        npos++;
                                }

                                if (!needle[npos])
                                        return o;
                        }
                }

                o = p;
        }

        return NULL;
}

gboolean
kozo_utf8_streqcasestrip (const char     *a,
                          const gunichar *b)
{
        gunichar unival, sc;
        const char *p;
        guint bpos;

        bpos = 0;
        for (p = e_util_unicode_get_utf8 (a, &unival);
             p && unival && b[bpos];
             p = e_util_unicode_get_utf8 (p, &unival)) {
                sc = stripped_char (unival);
                if (sc)
                        if (sc != b[bpos++])
                                return FALSE;
        }

        if ((p && unival) || b[bpos])
                return FALSE;

        return TRUE;
}
