/*
 * Bickley - a meta data management framework.
 * Copyright © 2008, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "bkl-marshal.h"
#include "bkl-orbiter.h"
#include "bkl-orbiter-dbus.h"

enum {
    ADDED,
    REMOVED,
    LAST_SIGNAL
};

G_DEFINE_TYPE (BklOrbiterDBus, bkl_orbiter_dbus, G_TYPE_OBJECT);

static gboolean bkl_orbiter_get_progress (BklOrbiterDBus *dbus,
                                          int            *OUT_total,
                                          int            *OUT_completed,
                                          int            *OUT_broken,
                                          GError        **error);
static gboolean bkl_orbiter_list_sources (BklOrbiterDBus *dbus,
                                          GPtrArray     **OUT_sources,
                                          GError        **error);
static gboolean bkl_orbiter_index_uri (BklOrbiterDBus *dbus,
                                       const char     *IN_uri,
                                       GError        **error);
static gboolean bkl_orbiter_remove_uri (BklOrbiterDBus *dbus,
                                        const char     *IN_uri,
                                        GError        **error);

#include "bkl-orbiter-glue.h"

static guint32 signals[LAST_SIGNAL] = {0, };

static gboolean
bkl_orbiter_get_progress (BklOrbiterDBus *dbus,
                          int            *OUT_total,
                          int            *OUT_completed,
                          int            *OUT_broken,
                          GError        **error)
{
    *OUT_total = 0;
    *OUT_completed = 0;
    *OUT_broken = 0;

    return TRUE;
}

static gboolean
bkl_orbiter_list_sources (BklOrbiterDBus *dbus,
                          GPtrArray     **OUT_sources,
                          GError        **error)
{
    GList *sources, *s;

    sources = bkl_orbiter_get_sources ();
    for (s = sources; s; s = s->next) {
    }

    /* Don't need to free the BklSource */
    g_list_free (sources);

    return TRUE;
}

static gboolean
bkl_orbiter_index_uri (BklOrbiterDBus *dbus,
                       const char     *IN_uri,
                       GError        **error)
{
    return TRUE;
}

static gboolean
bkl_orbiter_remove_uri (BklOrbiterDBus *dbus,
                        const char     *IN_uri,
                        GError        **error)
{
    return TRUE;
}

static void
bkl_orbiter_dbus_class_init (BklOrbiterDBusClass *klass)
{
    dbus_g_object_type_install_info (G_TYPE_FROM_CLASS (klass),
                                     &dbus_glib_bkl_orbiter_object_info);
    signals[ADDED] = g_signal_new ("source_added",
                                   G_TYPE_FROM_CLASS (klass),
                                   G_SIGNAL_RUN_FIRST |
                                   G_SIGNAL_NO_RECURSE, 0, NULL, NULL,
                                   bkl_marshal_VOID__STRING_STRING,
                                   G_TYPE_NONE, 2,
                                   G_TYPE_STRING, G_TYPE_STRING);
    signals[REMOVED] = g_signal_new ("source-removed",
                                     G_TYPE_FROM_CLASS (klass),
                                     G_SIGNAL_RUN_FIRST |
                                     G_SIGNAL_NO_RECURSE, 0, NULL, NULL,
                                     bkl_marshal_VOID__STRING_STRING,
                                     G_TYPE_NONE, 2,
                                     G_TYPE_STRING, G_TYPE_STRING);
}

static void
bkl_orbiter_dbus_init (BklOrbiterDBus *self)
{
}

BklOrbiterDBus *
bkl_orbiter_dbus_new (void)
{
    BklOrbiterDBus *dbus;

    dbus = g_object_new (BKL_TYPE_ORBITER_DBUS, NULL);

    return dbus;
}

void
bkl_orbiter_dbus_source_added (BklOrbiterDBus *dbus,
                               const char     *name,
                               const char     *uri)
{
    g_signal_emit (dbus, signals[ADDED], 0, name, uri);
}

void
bkl_orbiter_dbus_source_removed (BklOrbiterDBus *dbus,
                                 const char     *name,
                                 const char     *uri)
{
    g_signal_emit (dbus, signals[REMOVED], 0, name, uri);
}
