/*
 * Bickley - a meta data management framework.
 * Copyright © 2008, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA
 */

#include <string.h>

#include <glib.h>
#include <gio/gio.h>

#include "metadata.h"
#include "quick-audio-metadata.h"
#include "metadata-defines.h"

gboolean
bkl_task_quick_audio_get_metadata (GFile      *file,
                                   GFileInfo  *info,
                                   const char *mimetype,
                                   GHashTable *metadata)
{
    char *return_error = NULL;
    const char *data;
    int number;
    Metadata *md;

    /* FIXME: Return error needs to be replaced by GError */
    md = metadata_load (file, mimetype, &return_error);
    if (md == NULL) {
        g_free (return_error);
        return FALSE;
    }

    if (data = metadata_get_title (md)) {
        g_hash_table_insert (metadata, METADATA_AUDIO_TITLE, g_strdup (data));
    }

    /* FIXME: This only gets the first artist */
    if (data = metadata_get_artist (md, 0)) {
        g_hash_table_insert (metadata, METADATA_AUDIO_ARTIST, g_strdup (data));
    }

    if (data = metadata_get_album (md)) {
        g_hash_table_insert (metadata, METADATA_AUDIO_ALBUM, g_strdup (data));
    }

    if (data = metadata_get_genre (md)) {
        g_hash_table_insert (metadata, METADATA_AUDIO_GENRE, g_strdup (data));
    }

    if (data = metadata_get_year (md)) {
        g_hash_table_insert (metadata, METADATA_AUDIO_YEAR, g_strdup (data));
    }

    if (data = metadata_get_comment (md)) {
        g_hash_table_insert (metadata, METADATA_AUDIO_COMMENT, g_strdup (data));
    }

    number = metadata_get_duration (md);
    if (number > 0) {
        g_hash_table_insert (metadata, METADATA_AUDIO_DURATION,
                             g_strdup_printf ("%i", number));
    }

    number = metadata_get_track_number (md);
    if (number > 0) {
        g_hash_table_insert (metadata, METADATA_AUDIO_TRACK,
                             g_strdup_printf ("%i", number));
    }

    number = metadata_get_total_tracks (md);
    if (number > 0) {
        g_hash_table_insert (metadata, METADATA_AUDIO_TOTAL_TRACKS,
                             g_strdup_printf ("%i", number));
    }

    number = metadata_get_disc_number (md);
    if (number > 0) {
        g_hash_table_insert (metadata, METADATA_AUDIO_DISC,
                             g_strdup_printf ("%i", number));
    }

    /* FIXME: Need to get classical info, album art and replay gain
       Copyright, license etc */
    metadata_free (md);

    return TRUE;
}
